SystemOrganization addCategory: #'Kernel-Extensions'!
SystemOrganization addCategory: #'Kernel-Extensions-Tests'!

!SystemOrganizer class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 12:08'!
cleanUp

	Smalltalk forgetDoIts! !

ProtoObject subclass: #Null
	instanceVariableNames: 'originator fromMethod'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Kernel-Extensions'!

!Null commentStamp: 'kph 12/12/2007 11:16' prior: 0!
ifTrue:ifFalse: support is trickey due to the compiler inlining those calls. The current implementation is based upon my very limited knowledge - kph

this works
self assert: ([ null ifTrue: [1].2] value == 2)
this doesnt work
self assert: ([ null ifTrue: [ ^1 ].2] value == 2).

The above limitation requires #isKindOf: to be implemented else #printIt ceases to work.

====
Copyright 2000, by Nevin Pratt

In Smalltalk, as we all know, nil is the distinguished object which is typically used as the initial value of all
variables. It answers true to the message #isNil, and throws a "Does Not Understand" exception when almost
any other message is sent to it.

Do all dynamic languages have a similar concept, and a similar nil? No, they do not. In this article, I am going to
briefly compare Smalltalk's nil behavior to that of another dynamic language-Objective-C. Even though it has
been about six years since I have done any programming in Objective-C (because I switched to Smalltalk), I
have found some Objective-C techniques to be helpful and useful to my Smalltalk career. In particular, there are
certain situations where I actually prefer Objective-C's concept of a nil over Smalltalk's. This article explores
some of those situations.

If you answer nil in Objective-C, the nil that is returned is, in some respects, a lot like Smalltalk's nil, except
that instead of the nil generating exceptions when you message it, it just silently eats the messages. Thus, in
Objective-C, nil acts more like a "black hole". It is emptiness. It is nothingness. If you send anything to nil, nil
is all you get back. No exceptions. No return values. Nothing.

Obviously, if we wanted this same behavior in Smalltalk, we would simply override the #doesNotUnderstand:
instance message of the UndefinedObject class to just answer self. But would making this change be a good
idea in Smalltalk? No it wouldn't-but I'm getting ahead of myself in saying that. So, let's look at it a bit closer,
because in doing so it will eventually lead us to what I really do like!!

In Objective-C, nil didn't always have this "message eating" behavior. Brad Cox, the inventor of Objective-C,
originally gave nil behavior that more closely modeled Smalltalk's nil. That is, messaging nil originally
generated a runtime exception, as is evidenced in his release (via StepStone Corporation) of his "ICPack 101"
class library and accompanying compiler and runtime. Then, beginning with "ICPack 201" (and accompanying
compiler and runtime), this behavior was changed to the current "message eating" behavior. And, NeXT
Computers followed suit with their Objective-C implementation, in which they gave their nil a "message
eating" behavior, as did the Free Software Foundation with their GNU Objective-C compiler. But it wasn't
always that way.

So why did it change?

As you might guess, this change created two diverging camps among the programmers. On one side sat the
programmers that preferred the original "exception throwing" behavior, and on the other side sat the
programmers who preferred the new "message eating" behavior. And they each gave their best arguments to
try and illustrate why the philosophy of their side was superior to the other. It was a lively and interesting
debate that had no victors, other than the de-facto victor voiced by the compiler implementers themselves;
namely NeXT Computers, StepStone Corp, and later the FSF, all of which chose the "message eating"
behavior. But, the opinions voiced in the "pro" and "con" arguments were interesting, and especially interesting
in what they both actually agreed upon!!

Both sides agreed that the "message eating" behavior tended to create more elegant code!!

But of course, the "exception throwing" side responded by saying, "seemingly more elegant code, yes, but
potentially troublesome and unreliable", and they gave their reasons for asserting this. We will look at some of
those arguments, but first I will demonstrate how the "message eating" behavior can tend to make the code
more elegant looking.

Suppose, for example, that we wanted to find out the last telephone number that you dialed from your office
telephone, and we wanted to save this last number into a variable called, say, `lastNumber'. Suppose further
that we wanted to save it as a string so that we could display it in a GUI widget (as well as so we could use it
later). If you are the `person' in the message sequence below, would the message sequence to accomplish this
request be as follows? 

lastNumber := person office phone lastNumberDialed asString.
widget setStringValue: lastNumber.

Maybe.

But then, what if you don't have an office? Or, what if you have an office, but the office doesn't have a phone?
Or, what if it is new phone, and the phone has never been dialed yet? In any of these cases, using the
"exception throwing" nil convention, an exception will be thrown, thus potentially halting the program if an
exception handler hasn't been created to handle that exception.

But, what if nil has the "message eating" behavior? In this case, `lastNumber' could potentially have a final
value of nil, but the code above works just fine for this. Even passing nil as an argument to the
#setStringValue:1 method doesn't hurt, because the "message eating" nil convention is used by the widgetry as
well. It doesn't matter that the argument is nil. Everything still works, and there's no exception thrown, and no
immediately apparent strange side-effects (but we'll analyze that one some more later).

To contrast this, how then would you have to code it if nil has the "exception throwing" behavior? You would do
it similar to this:

  | tmp |
   tmp := person office.
   tmp notNil ifTrue: [tmp := tmp phone].
   tmp notNil ifTrue: [tmp := tmp lastNumberDialed].
   tmp notNil ifTrue: [lastNumber := tmp asString].
   widget setStringValue: lastNumber.

Yuck...all those explicit tests for nil are really ugly!! Of course, you could have instead wrapped the original code
in an exception handler, and thus avoided the nil tests, something like as follows:

   [lastNumber := person office phone lastNumberDialed asString.
   widget setStringValue: lastNumber]
      on: Object messageNotUnderstoodSignal do: [].

This looks a bit simpler than the previous example, but even this example contrasts poorly to the first example.
The first example of these three is much simpler!! You just "do it", without worrying about exceptions, exception
handlers, or explicit tests.

But, is this kind of code common? With the "exception throwing" nil, do we really end up typically testing for nil
like this, or else setting up exception handlers like this?

Yes, it is common. While the above example was contrived, let's look at a real-life example, from the
#objectWantingControl method of the VisualPart class of VisualWorks. The VisualPart class is a superclass of
the View class, and views in VisualWorks are coded to generally expect to have a collaborating controller
object for processing user input (mouse and keyboard events). Thus, the #objectWantingControl method is a
method of the view object, and it asks it's controller if it wants the user focus. If it does, #objectWantingControl
answers self, otherwise it answers nil. If nil is answered, then the view is considered to be read only, and will
not process user input. The actual implementation of #objectWantingControl is as follows:

objectWantingControl


   | ctrl |
   ctrl := self getController.
   ctrl isNil ifTrue: [^nil].
   " Trap errors occurring while searching for
   the object wanting control. "
   ^Object errorSignal
   handle: [:ex |
   Controller badControllerSignal
   raiseErrorString:
   'Bad controller in objectWantingControl']
   do: [ctrl isControlWanted ifTrue: [self] ifFalse: [nil]]

Notice that this method has both an explicit #isNil test, as well as an exception handler. How would this
method instead be written if the system had a "message eating" nil throughout? While there are a several
variations of possibilities, including at least one variation that is shorter (but not necessarily clearer), we would
probably write it as follows:

objectWantingControl
   self getController isControlWanted ifTrue: [^self].
   ^nil

Notice how much simpler it suddenly became. The programmer's intentions are much clearer. No #isNil checks,
no exceptions, no extra code to confuse the issue.

Furthermore, with the "exception throwing" nil, even when the code is written to avoid #isNil tests and/or
exception handlers, the coding style is usually altered in other ways to compensate. And, invariably, these style
alterations don't produce as elegant of code as if you had a "message eating" nil.

"Message eating" nil creates simpler, more elegant code. This was almost the unanimous opinion of both camps
of the Objective-C debate on this. But of course, the "exception throwing" camp argued that this "simpler" code
was also potentially more troublesome, and sometimes even buggy. And, they gave examples to illustrate. But
their examples also all seemed to fall into one of two arguments.

The first argument boiled down to the observation that, with a "message eating" nil, if a message sequence
produces nil as the final result, it is more difficult to determine exactly where the breakdown occurred. In other
words, what was the message that produced the first nil?

And, the response to this argument was: the programmer typically doesn't care what message produced the
first nil, and even if he did, he would explicitly test for it.

And of course, the response to this response was: the programmer should care, but typically won't care,
therefore the "message eating" nil is a feature which promotes bad programming habits.

And of course, this in turn illicited a response that essentially just disagreed with their conclusions and
challenged their statements, such as why the programmer should care, etc. 

And so the debate raged. But none-the-less, both sides seemed to admit that the "message eating" nil tended
to create simpler, more elegant looking code. And, the existing code base tended to substantiate this
conclusion. And simple code is good code, as long as it is also accurate code.

So, with a "message eating" nil, is the resulting code accurate? Or does the "message eating" nil tend to
introduce subtle bugs? To this question, the "exception throwing" crowd said it introduces subtle bugs, and they
gave specific examples. Interestingly enough, all of their examples that I looked at were with statically declared
variables, and those examples typically illustrated the platform dependent idiosyncracies that developed when a
nil was coerced into a static type. One specific example was illustrated via the following code snippet (which I
have modified to conform to Smalltalk syntax instead of Objective-C syntax):

   value := widget floatValue

In this example, if `value' is statically declared to be a variable of type float (floats are not objects in
Objective-C, but are instead native data types), and the #floatValue2 message returns nil instead of a valid
float number, then after the assignment has completed, `value' will equal zero on the Motorola M68K family of
processors, but on the Intel processor family, it ends up being a non-zero value, because of the peculiarities of
implicit casting of a nil to a native float datatype. This is clearly an undesirable result, and can lead to subtle
bugs.

But, while those examples might be relevant for Objective-C, they are totally irrelevant for Smalltalk. There is
no static declaration of variable types in Smalltalk, nor are there native data types (non-objects). It's a
non-issue in Smalltalk.

So, should the semantics of nil in Smalltalk be changed such that it eats messages? This is easy to do by
changing #doesNotUnderstand: to just answer self. Should we do it?

No, I don't think so. There has been too much code already written that is now expecting nil to throw
exceptions. To change the semantics of nil from "exception throwing" to "message eating" at this time would
likely break a large body of that code. It could be a very painful change, indeed.

Furthermore, even in Smalltalk, the first objection to a "message eating" nil still stands; to whit, in a given
message sequence whose final value is nil, it is difficult to determine what object first returned the nil. While it
is purely a subjective opinion as to how important that objection really is, I don't know how anyone could not
agree that it is indeed a valid objection. Minor perhaps (and perhaps not), but valid.

So, instead of modifying Smalltalk's nil, let's now briefly look at an alternative, that of sending back a
specialized Null object that has message-eating semantics. The first public document that I am aware of that
explored this alternative is Bobby Woolf's excellent white paper, "The Null Object Pattern"3, although earlier
works likely do exist. In that paper (which is now about five years old-almost an eternity in computer time), he
also uses the VisualPart example from above to illustrate his pattern. In fact, that is precisely why I also chose
to illustrate the results of using a "message eating" null via this same VisualPart example. That way, I could
keep things simple and consistent, without introducing too much additional code for all of the illustrations.

The "Null Object Pattern" essentially recommends the creation of a "do nothing" null object which implements
the same protocol as the original object, but does nothing in response to that protocol. For the VisualPart
example above, this pattern requires the creation of a class called NoController which implements the protocol
of a controller, but does nothing in response to it.

Doing nothing, however, means something special to a controller. For example, the NoController is expected to
answer false to the #isControlWanted message. Why is this important? Because clients of NoController expect
a boolean result to the #isControlWanted message, and they might in turn try sending #ifTrue: or #ifFalse: to
that result, and only booleans (and perhaps "message eating" nils) respond to #ifTrue: and #ifFalse. The
NoController has to return something that will respond to these boolean messages, or else the NoController is
not going to be plug-compatible with a real controller.

But, suppose we instead had #isControlWanted return a "message eating" nil? Or better yet, what if the
#getController method of the VisualPart returned a "message eating" nil? I believe everything would still "just
work", and that this also would be a simple way to generalize the "Null Object Pattern" of Woolf's paper.

Interestingly enough, in Woolf's paper, he describes an advantage of the "Null Object Pattern" by saying it...

...simplifies client code. Clients can treat real collaborators and null collaborators uniformly.
Clients normally don't know (and shouldn't care) whether they're dealing with a real or a null
collaborator. This simplifies client code, because it avoids having to write special testing code
to handle the null collaborator.

This testimony dovetails nicely with the NeXTSTEP community's assertion that the "message eating" nil
behavior of Objective-C appears to simplify code, as I have already demonstrated.

But, to implement the "Null Object Pattern", do we create a NoController class, and a NoOffice, and a NoPhone,
and a NoLastNumberDialed class? Where does it end? Indeed, this potential class explosion of the "Null Object
Pattern" is also mentioned by Woolf, as follows:

[One of] the disadvantages of the Null Object pattern [is]...class explosion. The pattern can
necessitate creating a new NullObject class for every new AbstractObject class.

A "message eating" nil would avoid this class explosion, as it is protocol-general instead of protocol-specific. I
personally feel that this difference is even a bit reminiscent of the static typing vs. dynamic typing differences
(and ensuing debates), as the following chart illustrates:

Static Typing vs. Dynamic Typing 

Should we allow any type of object to be
handled  (assigned to) this variable,
or only objects of a specific type?

Null Objects vs. Message Eating Nil
Should we allow any type of message to
be handled by (sent to) this object, or
only messages of a specific type
(protocol)?

I make no secret that I prefer dynamic typing over static typing. And, I also believe that often a general
"message eating" nil is more desirable than the more specific "Null Object Pattern", provided of course that the
"message eating" nil is implemented correctly. What follows is my implementation of a "message eating" nil,
which I call a null, which is an instance of my class Null.

Recall that the first objection against the null was that in a given message sequence whose final value is null, it
is difficult to determine what object first returned the null. How do we handle that objection?

Simple. Just ask it.

The Null class should have an originator instance variable that records who originally invoked the `Null new',
as well as a sentFromMethod instance variable that records from what method of the originator the `Null new'
was invoked.

But does that mean that the creator of the null must now tell the null so that the null can tell you? That sounds
like a lot of work!! And, what if someone forgets those extra steps?

Simple. Don't require the extra steps. Anybody should be able to send `Null new', and the Null class itself
should be able to figure this information out.

But that is not possible to do unless the Null class can somehow automatically determine who is calling one of
it's instance creation methods. In other words, we need to detect who the sender of the message is. How do we
do that? It is not part of standard Smalltalk!!

Well, here is how to do it in VisualWorks:

   Object>>sender
      ^thisContext sender sender receiver

   Object>>sentFromMethod
      ^thisContext sender sender selector


Now, in your other code, anytime you want to return a nil that also has message-eating semantics (which I call
a null), you use `^Null new' from your code instead of `^nil'. Then, your caller can easily discover the originator
of the null if it wishes to simply by asking.

If you are concerned about the potential proliferation of nulls with such a scheme, another trick you might try is
to create a default null using a `Default' class variable:

   Null class>>default
   Default == nil
   ifTrue:
   [Default := super new initialize.
      Default originator: Null.
      Default fromMethod: #default].
 ^Default

The default null can then later be accessed via `Null default' instead of `Null new'. I actually use this quite often
for automatic instance variable initialization in my abstract DomainModel class, which is the superclass of all of
my domain objects:

DomainObject>>initialize
   1 to: self class instSize
      do: [:ea |
         (self instVarAt: ea) isNil
            ifTrue:
               [self instVarAt: ea put: Null default]].
   ^self 

I have found that such a scheme does indeed simplify the domain logic, just as this article indicates that it
should. In fact, sometimes it has dramatically simplified things. And, I have never had any problems with this
scheme, as long as I have limited its use to the domain layer only. I have, however, had problems trying to
integrate some of these ideas into the GUI layer, and decided long ago that it was a bad idea in that layer.

My own implementation of the Null class was originally written in VisualAge, and was originally part of a much
larger domain-specific class library. This class library originally tried a number of ideas on an experimental
basis, to see if problems resulted from their use. The use of the Null pattern described in this article was one of
those experimental ideas. Even though it is actually a small idea, it's widespread use in the domain layer was
encouraged from my previous Objective-C experience, but I still didn't know if I would run into other subtle
issues while using it in Smalltalk. But I feel comfortable with it now in the domain layer (but not in the GUI
layer).

Some time after creating the class library I mentioned above, the entire class library was ported to GemStone,
and then finally the entire class library was moved to VisualWorks. A filein of the VisualWorks implementation
of the Null class follows. Email me at nevinop@xmission.com if you want either the VisualAge or GemStone
versions, and I'll try to dig them out.

If you instead decide to create your own Null class in VisualAge, another thing to realize is that #isNil is inlined
in VA (but not in VW). Thus, something like `Null new isNil' will always answer false in VA, even though your
Null>>isNil method explicitly answers true. Hence, in your domain code, with VA you probably want to create
an #isNull method and use that instead of #isNil. That is what I originally did, and that convention carried to the
GemStone version, but I have since broken that convention in the VisualWorks version.

1.As a sidebar, one could also argue about the appropriateness of a #setStringValue: method in this
example, and its implied limitation of only setting, or showing, strings, rather than having perhaps a
more generic #show: method that can show other types as well. To this, I have three things to say: 

first, consider the commonly used #show: method of the Transcript class in Smalltalk, and the argument
type it expects (strings) 
second, #setStringValue: is the actual method name used for TextField widgets in NeXTSTEP 
third, who cares, this is just a contrived example anyway. 

2.#floatValue also is an actual message implemented by TextFields under NeXTSTEP, just as the
#setStringValue: of the earlier code snippets is. 
3.Published in Pattern Languages of Program Design. Addison-Wesley, James Coplien and Douglas
Schmidt (editors). Reading, MA, 1995; http://www.awl.com/cseng/titles/0-201-60734-4. 
!

!Null class methodsFor: 'initialization' stamp: ''!
initialize
    Default := nil.
    Smalltalk at:#null put: (Null default).
! !

!Null class methodsFor: 'instance creation' stamp: ''!
default
    Default == nil ifTrue:[
        Default := super new initialize.
        Default originator:Null.
        Default fromMethod:#default
    ].
    ^ Default

 "self default"
! !

!Null class methodsFor: 'instance creation' stamp: 'kph 1/30/2008 16:52'!
freeSomeSpace

	self initialize! !

!Null class methodsFor: 'instance creation' stamp: ''!
new
    |tmp|
    tmp := super new.
    tmp originator:thisContext  sender receiver.
    tmp fromMethod:thisContext  sender selector.
    ^ tmp
! !

!Null methodsFor: 'as yet unclassified' stamp: 'kph 10/17/2007 14:39'!
<<< a

	^ self

! !

!Null methodsFor: 'comparing' stamp: 'kph 10/17/2006 07:52'!
= anObject
                             
^anObject isNil

! !

!Null methodsFor: 'printing' stamp: ''!
asString
    ^ ''
! !

!Null methodsFor: 'forwarding' stamp: ''!
doesNotUnderstand: aMessage

^self
! !

!Null methodsFor: 'accessing' stamp: ''!
fromMethod


^fromMethod
! !

!Null methodsFor: 'accessing' stamp: ''!
fromMethod: aSymbol
    ^ fromMethod := aSymbol
! !

!Null methodsFor: 'comparing' stamp: 'kph 10/17/2006 07:53'!
hash
 "Make the hash the same as a nil. Otherwise, each instance 
  of a null will have a different hash"

	^nil hash
! !

!Null methodsFor: 'testing' stamp: 'test 3/20/2007 16:18'!
ifNull: aBlock 

	^ aBlock value! !

!Null methodsFor: 'forwarding' stamp: 'kph 3/20/2007 16:40'!
in: aBlock

	^ self! !

!Null methodsFor: 'forwarding' stamp: ''!
initialize
! !

!Null methodsFor: 'testing' stamp: ''!
isDefaultNull

^self == self class default
! !

!Null methodsFor: 'printing' stamp: 'kph 12/12/2007 11:10'!
isKindOf: aClass 
	"Answer whether the class, aClass, is a superclass or class of the receiver."

	self class == aClass
		ifTrue: [^true]
		ifFalse: [^self class inheritsFrom: aClass]! !

!Null methodsFor: 'testing' stamp: 'kph 4/23/2008 23:51'!
isLiteral

^ true! !

!Null methodsFor: 'testing' stamp: 'test 3/20/2007 16:18'!
isNull

^self class == Null! !

!Null methodsFor: 'boolean interface' stamp: 'kph 12/12/2007 10:45'!
mustBeBoolean
	"Catches attempts to test truth of non-Booleans.  This message is sent from the VM.  The sending context is rewound to just before the jump causing this exception."

	^ self mustBeBooleanIn: thisContext sender! !

!Null methodsFor: 'boolean interface' stamp: 'kph 10/17/2006 09:07'!
mustBeBooleanIn: context

	"context is the where the non-boolean error occurred. I have no idea what I am doing here but it works for ifTrue: and ifTrue:ifFalse:"
 
	context nextInstruction.
 
	^ self! !

!Null methodsFor: 'accessing' stamp: ''!
originator

^originator
! !

!Null methodsFor: 'accessing' stamp: ''!
originator: aClass
   ^originator := aClass
! !

!Null methodsFor: 'printing' stamp: 'kph 12/12/2007 10:53'!
printOn: stream

	stream nextPutAll: self printString! !

!Null methodsFor: 'printing' stamp: 'kph 12/12/2007 10:51'!
printString
    "Print an representation of the receiver"

   ^ (self == (Null default)) ifTrue:[
        'Null default'
    ] ifFalse:[
        'Null(',self originator name,')'
    ]
! !

!Null methodsFor: 'printing' stamp: 'kph 4/23/2008 23:49'!
printStringLimitedTo: limit
	^ self printString! !

!Null methodsFor: 'forwarding' stamp: ''!
respondsTo: aSymbol

^true
! !

!Null methodsFor: 'comparing' stamp: 'kph 9/11/2007 16:13'!
size
	"enables null to be 'concatenated' into strings as an empty string"
	^ 0! !

!BlockContext methodsFor: '*kernel-extensions' stamp: 'kph 12/23/2007 23:40'!
= other

	self class == other class ifFalse: [^ false].
	self home receiver == other home receiver ifFalse: [^ false].
	self home selector == other home selector ifFalse: [^ false].
	^ self startpc == other startpc
! !

!BlockContext methodsFor: '*kernel-extensions' stamp: 'kph 10/19/2006 14:33'!
splitUp: aSequenceableCollection

   ^ aSequenceableCollection split: self! !

!DataStream class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 11:52'!
cleanUp

	^ self initialize! !

TestCase subclass: #SequenceableCollectionJoinSplitTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Extensions-Tests'!

!SequenceableCollectionJoinSplitTest methodsFor: 'as yet unclassified' stamp: 'kph 10/19/2006 12:09'!
array

	^ #( 1 2 3 4 )! !

!SequenceableCollectionJoinSplitTest methodsFor: 'as yet unclassified' stamp: 'kph 10/19/2006 12:09'!
arrayOfStrings

	^ #( 'Squeak is modern' 'open source' 'highly portable' 'fast'  'full-featured' )! !

!SequenceableCollectionJoinSplitTest methodsFor: 'as yet unclassified' stamp: 'kph 10/19/2006 12:08'!
eg

	^ 'Now is the time for all good men to come to the aid of the party'.
! !

!SequenceableCollectionJoinSplitTest methodsFor: 'the tests' stamp: 'kph 10/19/2006 12:03'!
testJoinTokensUsingCharacter

	self assert: ((self array joinUsing: $/) = '1/2/3/4')! !

!SequenceableCollectionJoinSplitTest methodsFor: 'the tests' stamp: 'kph 10/19/2006 12:03'!
testJoinTokensUsingString

	self assert: ((self array joinUsing: ', ') = '1, 2, 3, 4')! !

!SequenceableCollectionJoinSplitTest methodsFor: 'the tests' stamp: 'kph 10/19/2006 12:06'!
testJoinUsingArray

	| join | 
	
	self assert: ((join := self array joinUsing: #(10 11)) = #(1 10 11 2 10 11 3 10 11 4)).

	^ join! !

!SequenceableCollectionJoinSplitTest methodsFor: 'the tests' stamp: 'kph 10/19/2006 12:04'!
testJoinUsingLastOnArray

	self assert: ((self array joinUsing: ', ' last: ' and ') = '1, 2, 3 and 4')! !

!SequenceableCollectionJoinSplitTest methodsFor: 'the tests' stamp: 'kph 10/19/2006 12:04'!
testJoinUsingLastOnArrayOfStrings

	self assert: (
		(self arrayOfStrings joinUsing: ', ' last: ' and ') = 
		'Squeak is modern, open source, highly portable, fast and full-featured')! !

!SequenceableCollectionJoinSplitTest methodsFor: 'the tests' stamp: 'kph 10/19/2006 14:36'!
testSplitArray

	self assert: ((self testJoinUsingArray splitOn: #(10 11)) asArray = #( #(1) #(2) #(3) #(4)) )! !

!SequenceableCollectionJoinSplitTest methodsFor: 'the tests' stamp: 'kph 10/19/2006 15:15'!
testSplitBlock

	self assert: (('1,2.3-4' splitOn: [ :elem | ',.-' includes: elem ]) = #('1' '2' '3' '4') asOrderedCollection).
	self assert: (('1234' splitOn: [ :elem | elem asCharacter isAlphaNumeric]) = #('' '' '' '' '') asOrderedCollection).
	self assert: ((self array splitOn: [ :elem | elem \\ 2 = 0] ) = #(#(1) #(3) #()) asOrderedCollection).! !

!SequenceableCollectionJoinSplitTest methodsFor: 'the tests' stamp: 'kph 10/19/2006 14:36'!
testSplitString

	self assert: (((self eg splitOn: 'the') joinUsing: 'the') = self eg)! !

!StandardScriptingSystem class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 11:54'!
cleanUp

	^ self removeUnreferencedPlayers ! !

!Symbol class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 12:12'!
cleanUp

	3 timesRepeat: [
		Smalltalk garbageCollect.
		Symbol compactSymbolTable.
	].! !

!SystemDictionary methodsFor: '*kernel-extensions' stamp: 'kph 4/24/2008 03:45'!
globals
	"Smalltalk  inspectGlobals"
	
	| associations |
	
	associations := OrderedCollection new.
	
	self keysAndValuesDo: [ :key :value |
		
		((value isKindOf: Class) or: [ value isKindOf: Trait ])
		ifFalse: [ associations add: (key -> value) ]
	].

	^ associations asSortedCollection: [ :a :b | a key <= b key ].
  ! !

!UndefinedObject methodsFor: '*kernel-extensions-null' stamp: 'kph 4/12/2007 08:26'!
orNull

	^ Null default! !

!MethodChangeRecord class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 12:10'!
cleanUp

	MethodChangeRecord allInstancesDo: [:each | each noteNewMethod: nil].
! !

!ClassDescription methodsFor: '*kernel-extensions' stamp: 'kph 11/18/2007 02:42'!
onClassChanged

	"a hook, called when the class has been edited"

 ! !

!SMSqueakMap class methodsFor: '*kernel-extensions' stamp: 'kph 11/1/2007 02:37'!
freeSomeSpace

	self default clearCaches! !

!WriteStream methodsFor: '*kernel-extensions' stamp: 'kph 10/9/2007 01:40'!
<< aCollection
	"we want a readable version of nextPutAll however it may be difficult to fully recreate nextPutAll:
	for all the different types of stream. Rather then simply send to nextPutAll:
	we handle the String (or ByteArray) argument
	as fast as possible - the rest we delegate to putOn: This means that we handle single characters and bytes 
	whereas nextPutAll: is only for sequencable collections.
	.
	Note this may not work in every case that nextPutAll: does subject to extensive testing, 
	but it should work in the important cases"

	| newEnd |
	collection class == aCollection class ifFalse:
		[ aCollection putOn: self. ^ self ].

	newEnd _ position + aCollection size.
	newEnd > writeLimit ifTrue:
		[self growTo: newEnd + 10].

	collection replaceFrom: position+1 to: newEnd  with: aCollection startingAt: 1.
	position _ newEnd.

! !

!Magnitude methodsFor: '*kernel-extensions' stamp: 'kph 9/27/2007 22:10'!
putOn: aStream

	(aStream isBinary ifTrue: [ self asByteArray ] ifFalse: [ self asString]) putOn: aStream
	
 ! !

!SystemVersion methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 06:58'!
version
	"copy to impart immutability"
	^version copy! !

!TraitBehavior methodsFor: '*kernel-extensions' stamp: 'kph 11/18/2007 02:30'!
onClassChanged

	"a hook, called when the class has been edited"

 ! !

!Collection methodsFor: '*kernel-extensions' stamp: 'kph 1/1/2008 16:58'!
reject: selectBlock thenDo: doBlock 
	"Utility method to improve readability."
	
	^ self do: [:each | (selectBlock value: each) ifFalse: [doBlock value: each]]! !

!Collection methodsFor: '*kernel-extensions' stamp: 'kph 12/12/2007 12:55'!
removeAll
	"the slow way"
	^ self removeAllSuchThat: [ :each | true ]! !

!Collection methodsFor: '*kernel-extensions' stamp: 'kph 1/1/2008 16:55'!
select: selectBlock thenDo: doBlock 
	"Utility method to improve readability."
	
	^ self do: [:each | (selectBlock value: each) ifTrue: [doBlock value: each]]! !

!RequiredSelectors class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 11:58'!
cleanUp

	^ self current ensureClean! !

!EventManager class methodsFor: '*kernel-extensions' stamp: 'kph 8/23/2007 15:47'!
cleanUp
	
	 
	 ActionMaps := nil.
	
	 ! !

!ByteArray methodsFor: '*kernel-extensions' stamp: 'kph 9/27/2007 21:28'!
putOn: aStream

	aStream nextPutAll: self! !

!Character methodsFor: '*kernel-extensions' stamp: 'kph 10/19/2006 11:56'!
useToJoin: aCollection

	^ String streamContents: [ :stream |
		aCollection
			do: [ :each | stream nextPutAll: each asString ]
			separatedBy: [ stream nextPut: self ] ]! !

!TCompilingBehavior methodsFor: '*kernel-extensions' stamp: 'kph 11/18/2007 02:30'!
onClassChanged

	"a hook, called when the class has been edited"

 ! !

!String methodsFor: '*kernel-extensions' stamp: 'kph 10/19/2006 11:54'!
useToJoin: aCollection

	^ String streamContents: [ :stream | 
		aCollection
			do: [ :each | stream nextPutAll: each asString ]
			separatedBy: [ stream nextPutAll: self ] ]! !

!String methodsFor: '*kernel-extensions' stamp: 'kph 12/17/2007 20:58'!
withoutTrailing: charOrArray
	"Return a copy of the receiver from which trailing slash has been trimmed."
	
	self size = 0 ifTrue: [ ^ self ].

	charOrArray isCollection ifFalse: [ 
		self last = charOrArray  ifTrue: [ ^self copyFrom: 1 to: self size - 1 ].
	] ifTrue: [ 
		(charOrArray includes: self last) ifTrue: [ ^self copyFrom: 1 to: self size - 1 ]
	].! !

!ProcessBrowser class methodsFor: '*kernel-extensions-override' stamp: 'kph 10/2/2007 14:05'!
prototypicalToolWindow
	"Answer a window representing a prototypical instance of the receiver"

	^ self new asPrototypeInWindow extent: 400@300; yourself! !

!Stream methodsFor: '*kernel-extensions' stamp: 'kph 9/27/2007 21:53'!
<< items

 	items putOn: self.
	
	^ self! !

!Behavior class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 12:09'!
cleanUp
	^ self flushObsoleteSubclasses ! !

!Behavior methodsFor: '*kernel-extensions' stamp: 'kph 10/17/2007 06:45'!
allSelectorsBelow: topClass 
	| coll |
	coll := OrderedCollection new.
	self withAllSuperclasses do: 
			[:aClass | 
			aClass = topClass ifTrue: [^ coll asIdentitySet ] ifFalse: [coll addAll: aClass selectors]].
		
	^ coll asIdentitySet
	! !

!Behavior methodsFor: '*kernel-extensions-override' stamp: 'kph 11/18/2007 02:34'!
compile: code notifying: requestor 
	"Compile the argument, code, as source code in the context of the 
	receiver and insEtall the result in the receiver's method dictionary. The 
	second argument, requestor, is to be notified if an error occurs. The 
	argument code is either a string or an object that converts to a string or 
	a PositionableStream. This method also saves the source code."
	
	| methodAndNode |
	methodAndNode  := self
		compile: code "a Text"
		classified: nil
		notifying: requestor
		trailer: self defaultMethodTrailer
		ifFail: [^nil].
	methodAndNode method putSource: code fromParseNode: methodAndNode node inFile: 2
			withPreamble: [:f | f cr; nextPut: $!!; nextChunkPut: 'Behavior method'; cr].
	self addSelector: methodAndNode selector withMethod: methodAndNode method notifying: requestor.
	self onClassChanged.
	^ methodAndNode selector! !

!Behavior methodsFor: '*kernel-extensions' stamp: 'kph 11/18/2007 02:30'!
onClassChanged

	"a hook, called when the class has been edited"

 ! !

!CompiledMethod methodsFor: '*kernel-extensions' stamp: 'kph 4/24/2008 02:49'!
browse

	self methodReference browse.! !

!MethodReference methodsFor: '*kernel-extensions' stamp: 'kph 4/24/2008 02:38'!
browse

	ToolSet browse: self actualClass selector: self methodSymbol.! !

!MethodReference methodsFor: '*kernel-extensions' stamp: 'kph 4/24/2008 02:22'!
remove

	self isLocalSelector ifFalse: [ self error: 'not a local selector' ]. 
		
	self actualClass removeSelector: self methodSymbol ! !

!Object methodsFor: '*kernel-extensions' stamp: 'kph 1/27/2008 19:21'!
askFor: selector 

    "returns true or false"
	
	^ (self askFor: selector ifAbsent: nil) == true! !

!Object methodsFor: '*kernel-extensions' stamp: 'kph 10/17/2007 14:01'!
askFor: selector ifAbsent: aBlock

   "enables a default value to be specified in order to be tolerant of potentially missing methods
	
	e.g.
	(myPoint askFor: #originOffset) ifAbsent: [ 0@0 ].
	"

	^ (self class canUnderstand: selector) ifFalse: [ aBlock value ] ifTrue: [self perform: selector]! !

!Object methodsFor: '*kernel-extensions' stamp: 'mtf 1/26/2008 23:34'!
deprecated
	"Warn that the sending method has been deprecated."

	Preferences showDeprecationWarnings ifTrue:
		[Deprecation signal: thisContext sender printString, ' has been deprecated.']! !

!Object methodsFor: '*kernel-extensions-null' stamp: 'kph 9/6/2007 23:31'!
ifNull: aBlock

	^ self! !

!Object methodsFor: '*kernel-extensions-null' stamp: 'kph 9/6/2007 23:33'!
isNull

	^ false! !

!Object methodsFor: '*kernel-extensions' stamp: 'mtf 1/26/2008 23:52'!
log
	"This method provides the univeral entry point fo all logging mechanisms"
	
	"Options:
	1. Null for null logging
	2. A LogRouter instance wih a FrameworkAdaptor.
	3. CurrentLog a process local variable supplying a LogRouter"
	
	^ (Smalltalk at: #CurrentLog ifAbsent: [ Null default ]) value
		sender: thisContext sender; beginEntry; yourself! !

!Object methodsFor: '*kernel-extensions-null' stamp: 'kph 4/12/2007 08:27'!
orNull

	^ self! !

PackageInfo subclass: #KernelExtensionsInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Extensions'!

!ChangeSet class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 12:05'!
cleanUp

	| projectChangeSetNames |
	projectChangeSetNames := Project allSubInstances collect: [:proj | proj changeSet name].

	self removeChangeSetsNamedSuchThat:
		[:cs | (projectChangeSetNames includes: cs) not].! !

!SmalltalkImage class methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 12:11'!
cleanUp

	Undeclared removeUnreferencedKeys.
 	self current fixObsoleteReferences.! !

!SmalltalkImage methodsFor: '*kernel-extensions' stamp: 'kph 10/17/2007 14:18'!
cleanUpAll

	"this method runs #cleanUp on all classes which implement it
	It is intended to reset the image to a clean but non-operational state. i.e
	Servers will be stopped, caches flushed etc.
	
	see #freeSpace for 'non-destructive' version"
	
	self freeSpace.

	Smalltalk do: [ :class | class askFor: #cleanUp ]

! !

!SmalltalkImage methodsFor: '*kernel-extensions' stamp: 'kph 10/17/2007 14:17'!
freeSpace

	Smalltalk do: [ :class | class askFor: #freeSomeSpace ]
! !

!Utilities class methodsFor: '*kernel-extensions' stamp: 'kph 10/17/2007 15:36'!
cleanseOtherworldlySteppers
	"category  should be 'miscellaneous' "
	
	"If the current project is a morphic one, then remove from its steplist those morphs that are not really in the world"

	| old delta |
	Smalltalk isMorphic ifTrue:
		[old _ self currentWorld stepListSize.
		self currentWorld steppingMorphsNotInWorld do: [:m | m delete].
		self currentWorld cleanseStepList.
		(delta _ (old - self currentWorld stepListSize)) > -1 ifTrue:
			[self log kernel << delta <<< ' morphs removed from steplist' ]]

	"Utilities cleanseOtherworldlySteppers"! !

!Utilities class methodsFor: '*kernel-extensions' stamp: 'kph 10/17/2007 13:50'!
freeSomeSpace
	"#freeSomeSpace is a new protocol for signalling those methods
	which can be used to clean up an image before saving or deployment"

	self cleanseOtherworldlySteppers! !

!Set methodsFor: '*kernel-extensions' stamp: 'kph 12/23/2007 17:51'!
likeOrAdd: newObject
	"Include newObject as one of the receiver's elements, but only if
	not already present. Answer the old object."

	| index |
	newObject ifNil: [self error: 'Sets cannot meaningfully contain nil as an element'].
	index := self findElementOrNil: newObject.
	^ (array at: index) ifNil: [self atNewIndex: index put: newObject].
	 ! !

!SequenceableCollection class methodsFor: '*kernel-extensions' stamp: 'kph 9/27/2007 23:30'!
<< blockWithArg
 
	blockWithArg isBlock ifTrue: [ ^ self streamContents: blockWithArg ].
	 
	^ (WriteStream on: (self new: 100)) << blockWithArg
	
	"
	 (String << 'hello' << 'world') contents
	
	String << [:str | str << 'hello' << 'world' ]
	"
 ! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 06:53'!
<= otherCollection

"comparison for multi key sorts. note how boolean and nil IS supported for sorting"

self with: otherCollection do: [ :a :b | 
	
	(a == true and: [ b == false ]) ifTrue: [ ^ false ].
	(a == false and: [ b == true ]) ifTrue: [ ^ true ].
	
	a isNil ifTrue: [ ^ true ].
	b isNil ifTrue: [ ^ false ].
			
			a ~= b ifTrue: [ ^ a <= b ].
			
		].
	
^ false

"
#(1 2) <= #(1 3)
#(1 3) <= #(1 1)
#(1 2 false) <= #(1 2 true)
#(1 2 true) <= #(1 2 false)
#(1 2 true) <= #(1 2 nil)
"! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 06:48'!
copyReplacing: oldObject with: newObject 

	"unfortunately copyReplaceAll:with: is not consistent with the behaviour of replaceAll:with:"
	
	^ self copy replacing: oldObject with: newObject ! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 9/4/2007 14:20'!
firstOrNil
	"Answer the first element of the receiver"

	^ self at: 1 ifAbsent: [ nil ]! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 10/19/2006 15:31'!
joinUsing: joiner

 "joiner - character, string or sequenceable collection
  returns collection of the same collection class as 'joiner', or a String "
 
^ joiner useToJoin: self! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 10/19/2006 15:28'!
joinUsing: joiner last: last

  " #(1 2 3 4) joinUsing: ', ' last: 'and'. => '1, 2, 3 and 4 "

^ last useToJoin: (Array with: (joiner useToJoin: (self allButLast)) with: self last).! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 2/4/2008 18:37'!
lastOrNil
	"Answer the first element of the receiver"

	^ self at: (self size) ifAbsent: [ nil ]! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 9/27/2007 21:18'!
putOn: aStream

	self do: [ :each | each putOn: aStream ]! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 06:49'!
replaceAll: oldObject with: newObject 

	"DEPRECATE - use #replacing:with:"

	"Replace all occurences of oldObject with newObject"
	| index |
	index := self
				indexOf: oldObject
				startingAt: 1
				ifAbsent: [0].
	[index = 0]
		whileFalse: 
			[self at: index put: newObject.
			index := self
						indexOf: oldObject
						startingAt: index + 1
						ifAbsent: [0]]! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 1/18/2008 06:48'!
replacing: oldObject with: newObject 
	"Replace all occurences of oldObject with newObject"
	| index |
	index := self
				indexOf: oldObject
				startingAt: 1
				ifAbsent: [0].
	[index = 0]
		whileFalse: 
			[self at: index put: newObject.
			index := self
						indexOf: oldObject
						startingAt: index + 1
						ifAbsent: [0]]! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 10/13/2007 02:23'!
split: aBlock

	"split a collection into a sequence of sub-collections based upon
	 the true/false value returnd from the block"
	
	| result position |

	result := OrderedCollection new.
	position := 1.
	 
	self withIndexDo: [ :element :idx | 

		(aBlock value: element) ifTrue: [ result add: (self copyFrom: position to: idx - 1).
										 position := idx + 1  ]

	].

	result add: (self copyFrom: position to: self size).
	
	^ result

! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 10/17/2007 03:25'!
splitOn: splitter

 	"splitter - can be a sub-sequence, or aBlock"

	^ splitter splitUp: self! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 10/17/2007 03:29'!
splitUp: aSequencableCollection
 
	"split collection on sub-collections"
	
	| result position oldPosition |

	result := OrderedCollection new.
	position := 1.
	oldPosition := position.
	position := aSequencableCollection indexOfSubCollection: self startingAt: position.

	[ position > 0 ] whileTrue: [
		result add: (aSequencableCollection copyFrom: oldPosition to: position - 1).
		position := position + self size.
		oldPosition := position.
		position := aSequencableCollection indexOfSubCollection: self startingAt: position.
	].

	result add: (aSequencableCollection copyFrom: oldPosition to: aSequencableCollection size).

	^ result

! !

!SequenceableCollection methodsFor: '*kernel-extensions' stamp: 'kph 10/19/2006 12:00'!
useToJoin: aCollection

	^ self class streamContents: [ :stream |
		aCollection
			do: [ :each | stream nextPut: each ]
			separatedBy: [ stream nextPutAll: self ] ]! !

!TPureBehavior methodsFor: '*kernel-extensions' stamp: 'kph 11/18/2007 02:30'!
onClassChanged

	"a hook, called when the class has been edited"

 ! !

!AppRegistry class methodsFor: '*kernel-extensions' stamp: 'ar 9/27/2005 21:48'!
removeObsolete
	"AppRegistry removeObsoleteClasses"
	self registeredClasses copy do:[:cls| 
		(cls class isObsolete or:[cls isBehavior and:[cls isObsolete]]) 
			ifTrue:[self unregister: cls]].
	self subclasses do:[:cls| cls removeObsolete].! !
Null initialize!
